--TerrainMap uses the "Compass Tiles" resources to build a world map

-- to do
-- adjust NSOffset,EWOffset, XOffSet, YOffSet for region 4
-- fix missing tiles in region 4
-- capture new zoomed tiles for region 4

-- annotation caching is currently turned off due to an intermittant bug that fails to load annotations
-- changing region and back clears the bug. not sure how to reliably reproduce it yet

importPath=string.gsub(getfenv(1)._.Name,"%.Main","").."."
resourcePath=string.gsub(importPath,"%.","/").."Resources/"
tilePath=string.gsub(importPath,"%.","/").."Tiles/"

import "Turbine.UI";
import "Turbine.UI.Lotro";
import (importPath.."Class");
import (importPath.."Table");
import (importPath.."VindarPatch");

displayWidth, displayHeight = Turbine.UI.Display.GetSize();
UIFont=Turbine.UI.Lotro.Font.Verdana16

string.encode=function(str)
	-- replace all non-alpha characters with a % escaped hex code
	local ret=""
	for i,v in ipairs({str:byte(1,-1)}) do
		if (v < 48) or (v>57 and v<65) or (v>90 and v<97) or (v > 125) then
			ret = ret .. "%" .. string.sub("00"..string.format("%x",tonumber(v)),-2)
		else
			ret = ret .. string.char(v)
		end
	end
	return ret
end
string.decode=function(str)
	if str==nil then str="" end
	local ret=""
	while string.len(str)~=0 do
		local char=string.sub(str,1,1)
		if char~="%" then
			ret=ret..char
			str=string.sub(str,2,-1)
		else
			local char=string.char(tonumber("0x"..string.sub(str,2,3)))
			ret=ret..char
			str = string.sub(str,4,-1)
		end
	end
	return ret;
end
string.split=function(str,separator)
	local tmpArray={};
	local tmpElem;
	str=tostring(str);
	if string.find("^$()%.[]*+-?",separator,1,true)~=nil then separator="%"..separator end
	for tmpElem in string.gmatch(str,"[^"..separator.."]*") do
		table.insert(tmpArray,tmpElem);
	end
	local tmpIndex=1;
	while tmpIndex<=#tmpArray do
		if tmpIndex==#tmpArray and tmpArray[tmpIndex]=="" then
			table.remove(tmpArray,tmpIndex);
		else
			if tmpArray[tmpIndex]=="" and tmpArray[tmpIndex+1]~="" then
				table.remove(tmpArray,tmpIndex);
			else
				tmpIndex=tmpIndex+1;
			end
		end
	end
	return tmpArray;
end
function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end
if (tonumber("1,000")==1) then
	function euroNormalize(value)
		return tonumber((string.gsub(value,"%.",",")));
	end
else
	function euroNormalize(value)
		return tonumber((string.gsub(value,",",".")));
	end
end
function SaveData()
	Settings.Language=language
	Settings.Opacity=mapWindow:GetOpacity()
	Settings.TopPercent=mapWindow:GetTop()/displayHeight*100
	Settings.LeftPercent=mapWindow:GetLeft()/displayWidth*100
	Settings.HeightPercent=mapWindow:GetHeight()/displayHeight*100
	Settings.WidthPercent=mapWindow:GetWidth()/displayWidth*100
	Settings.IconLeftPercent=icon:GetLeft()/displayWidth*100
	Settings.IconTopPercent=icon:GetTop()/displayHeight*100
	Settings.LastRegion=mapWindow.Region:GetSelectedChoice()


	local VScrollVal=mapWindow.vGridScroll:GetValue()
	local HScrollVal=mapWindow.hGridScroll:GetValue()
	local Zoom=2^(mapWindow.ZoomLevel-1)
	if mapWindow.ZoomLevel/2==math.floor(mapWindow.ZoomLevel/2) then
		Zoom=Zoom/2
	end
	local x=(HScrollVal/200*Zoom+mapWindow.mapViewPort:GetWidth()/400*Zoom+1)*.8
	local y=(VScrollVal/200*Zoom+mapWindow.mapViewPort:GetHeight()/400*Zoom+1)*.8
	Settings.LastNS=Grid[Settings.LastRegion].NSOffset-y
	Settings.LastEW=Grid[Settings.LastRegion].EWOffset+x

	PatchDataSave(Turbine.DataScope.Account,"TerrainMapSettings",Settings);
	PatchDataSave(Turbine.DataScope.Account,"TerrainMapAnnotations",Annotations);
end
function LoadData()
	Settings=PatchDataLoad(Turbine.DataScope.Account, "TerrainMapSettings");
	if Settings==nil then
		Settings={}
	else
		Settings.Opacity=euroNormalize(Settings.Opacity)
		Settings.TopPercent=euroNormalize(Settings.TopPercent)
		Settings.LeftPercent=euroNormalize(Settings.LeftPercent)
		Settings.WidthPercent=euroNormalize(Settings.WidthPercent)
		Settings.HeightPercent=euroNormalize(Settings.HeightPercent)
	end
if Settings.LastRegion==nil then
	Settings.LastRegion=1
	Settings.LastNS=-15
	Settings.LastEW=-104
else
	if Settings.LastNS==nil then Settings.LastNS=0 end
	if Settings.LastEW==nil then Settings.LastEW=0 end
end

	if Settings.Language==nil then
		local tmpLanguage=Turbine.Engine:GetLanguage()
		if tmpLanguage==Turbine.Language.German then
			language=3
		elseif tmpLanguage==Turbine.Language.French then
			language=2
		elseif tmpLanguage==Turbine.Language.Russian then
			language=4 -- language=4 for russian was never completed, there are no string values so everything defaults back to english
		else
			language=1
		end
	else
		language=Settings.Language
	end
	if Settings.LoadMinimized==nil then
		Settings.LoadMinimized=false
	end
	if Settings.ShowIcon==nil then
		Settings.ShowIcon=true
	end
	if Settings.Opacity==nil then
		Settings.Opacity=1
	end
	if Settings.TopPercent==nil then
		Settings.TopPercent=0
	end
	if Settings.LeftPercent==nil then
		Settings.LeftPercent=0
	end
	if Settings.WidthPercent==nil then
		Settings.WidthPercent=100
	end
	if Settings.HeightPercent==nil then
		Settings.HeightPercent=100
	end
	if Settings.ShowAnnotations==nil then
		Settings.ShowAnnotations=true
	end
	if Settings.Filter==nil then
		Settings.Filter={}
	end
--	Annotations=PatchDataLoad(Turbine.DataScope.Account, "TerrainMapAnnotations");
	if Annotations==nil then
		Annotations={}
	end
end
LoadData();
import (importPath.."DebugWindow");
import (importPath.."Grid");
import (importPath.."Strings");
import (importPath.."DropDownList");
import (importPath.."Icon");
import (importPath.."Options");
import (importPath.."PopUpDialog");
import (importPath.."Search");
import (importPath.."AnnotIcons");
annotHover=Turbine.UI.ContextMenu()
annotHover.Display=function(id,annotRow,annotCol,X,Y)
	local annotList=annotHover:GetItems()
	annotList:Clear()
	local region=mapWindow.Region:GetSelectedChoice()
	local delta=16
	if mapWindow.Zoom:GetValue()==2 then delta=32 end
	-- clear list
	-- add an entry for THIS annotation
	-- add an entry for any annotation in same grid where id<>id and abs(sender.X-x)<16 and abs(sender.Y-y)<16
	for k,v in ipairs(Annotations[region][annotRow][annotCol]) do
		if math.abs(X-v.X)<delta and math.abs(Y-v.Y)<delta then
			local menuItem
			local name=v.Name
			if name=="" then name=v.Description end
			if annotIcon[v.Type]==nil then
				menuItem=Turbine.UI.MenuItem(name.." (...)")
			else
				menuItem=Turbine.UI.MenuItem(name.." ("..Resource[language][annotIcon[v.Type].StringID]..")")
			end
			menuItem.Name=name
			menuItem.Description=v.Description
			menuItem.ID=v.id
			menuItem.Click=function(sender,args)
				-- show details
				if mapWindow.PopUp~=nil then
					mapWindow.PopUp:SetVisible(false)
					mapWindow.PopUp=nil
				end
				mapWindow.PopUp=PopUpDialog(sender.Name,sender.Description,4,"OK")
				mapWindow.PopUp:SetZOrder(mapWindow:GetZOrder()+1)
			end
			annotList:Add(menuItem)
		end
	end
	if X<delta and annotCol>0 then
		local X2=200+X
		if Annotations[region][annotRow][annotCol-1]~=nil then
			for k,v in ipairs(Annotations[region][annotRow][annotCol-1]) do
				if math.abs(X2-v.X)<delta and math.abs(Y-v.Y)<delta then
					local menuItem
					local name=v.Name
					if name=="" then name=v.Description end
					if annotIcon[v.Type]==nil then
						menuItem=Turbine.UI.MenuItem(name.." (...)")
					else
						menuItem=Turbine.UI.MenuItem(name.." ("..Resource[language][annotIcon[v.Type].StringID]..")")
					end
					menuItem.Name=name
					menuItem.Description=v.Description
					menuItem.ID=v.id
					menuItem.Click=function(sender,args)
						-- show details
						if mapWindow.PopUp~=nil then
							mapWindow.PopUp:SetVisible(false)
							mapWindow.PopUp=nil
						end
						mapWindow.PopUp=PopUpDialog(sender.Name,sender.Description,4,"OK")
						mapWindow.PopUp:SetZOrder(mapWindow:GetZOrder()+1)
					end
					annotList:Add(menuItem)
				end
			end
		end
		if Y<delta and annotRow>0 and Annotations[region][annotRow-1]~=nil then
			local Y2=200+Y
			if Annotations[region][annotRow-1][annotCol-1]~=nil then
				for k,v in ipairs(Annotations[region][annotRow-1][annotCol-1]) do
					if math.abs(X2-v.X)<delta and math.abs(Y2-v.Y)<delta then
						local menuItem
						local name=v.Name
						if name=="" then name=v.Description end
						if annotIcon[v.Type]==nil then
							menuItem=Turbine.UI.MenuItem(name.." (...)")
						else
							menuItem=Turbine.UI.MenuItem(name.." ("..Resource[language][annotIcon[v.Type].StringID]..")")
						end
						menuItem.Name=name
						menuItem.Description=v.Description
						menuItem.ID=v.id
						menuItem.Click=function(sender,args)
						-- show details
							if mapWindow.PopUp~=nil then
								mapWindow.PopUp:SetVisible(false)
								mapWindow.PopUp=nil
							end
							mapWindow.PopUp=PopUpDialog(sender.Name,sender.Description,4,"OK")
							mapWindow.PopUp:SetZOrder(mapWindow:GetZOrder()+1)
						end
						annotList:Add(menuItem)
					end
				end
			end
		end
		if (200-Y)<delta and Annotations[region][annotRow+1]~=nil then
			local Y2=Y-200
			if Annotations[region][annotRow+1][annotCol-1]~=nil then
				for k,v in ipairs(Annotations[region][annotRow+1][annotCol-1]) do
					if math.abs(X2-v.X)<delta and math.abs(Y2-v.Y)<delta then
						local menuItem
						local name=v.Name
						if name=="" then name=v.Description end
						if annotIcon[v.Type]==nil then
							menuItem=Turbine.UI.MenuItem(name.." (...)")
						else
							menuItem=Turbine.UI.MenuItem(name.." ("..Resource[language][annotIcon[v.Type].StringID]..")")
						end
						menuItem.Name=name
						menuItem.Description=v.Description
						menuItem.ID=v.id
						menuItem.Click=function(sender,args)
						-- show details
							if mapWindow.PopUp~=nil then
								mapWindow.PopUp:SetVisible(false)
								mapWindow.PopUp=nil
							end
							mapWindow.PopUp=PopUpDialog(sender.Name,sender.Description,4,"OK")
							mapWindow.PopUp:SetZOrder(mapWindow:GetZOrder()+1)
						end
						annotList:Add(menuItem)
					end
				end
			end
		end
	end
	if (200-X)<delta then
		local X2=X-200
		if Annotations[region][annotRow][annotCol+1]~=nil then
			for k,v in ipairs(Annotations[region][annotRow][annotCol+1]) do
				if math.abs(X2-v.X)<delta and math.abs(Y-v.Y)<delta then
					local menuItem
					local name=v.Name
					if name=="" then name=v.Description end
					if annotIcon[v.Type]==nil then
						menuItem=Turbine.UI.MenuItem(name.." (...)")
					else
						menuItem=Turbine.UI.MenuItem(name.." ("..Resource[language][annotIcon[v.Type].StringID]..")")
					end
					menuItem.Name=name
					menuItem.Description=v.Description
					menuItem.ID=v.id
					menuItem.Click=function(sender,args)
						-- show details
						if mapWindow.PopUp~=nil then
							mapWindow.PopUp:SetVisible(false)
							mapWindow.PopUp=nil
						end
						mapWindow.PopUp=PopUpDialog(sender.Name,sender.Description,4,"OK")
						mapWindow.PopUp:SetZOrder(mapWindow:GetZOrder()+1)
					end
					annotList:Add(menuItem)
				end
			end
		end
		if Y<delta and annotRow>0 and Annotations[region][annotRow-1]~=nil then
			if Annotations[region][annotRow-1][annotCol+1]~=nil then
				local Y2=200+Y
				for k,v in ipairs(Annotations[region][annotRow-1][annotCol+1]) do
					if math.abs(X2-v.X)<delta and math.abs(Y2-v.Y)<delta then
						local menuItem
						local name=v.Name
						if name=="" then name=v.Description end
						if annotIcon[v.Type]==nil then
							menuItem=Turbine.UI.MenuItem(name.." (...)")
						else
							menuItem=Turbine.UI.MenuItem(name.." ("..Resource[language][annotIcon[v.Type].StringID]..")")
						end
						menuItem.Name=name
						menuItem.Description=v.Description
						menuItem.ID=v.id
						menuItem.Click=function(sender,args)
						-- show details
							if mapWindow.PopUp~=nil then
								mapWindow.PopUp:SetVisible(false)
								mapWindow.PopUp=nil
							end
							mapWindow.PopUp=PopUpDialog(sender.Name,sender.Description,4,"OK")
							mapWindow.PopUp:SetZOrder(mapWindow:GetZOrder()+1)
						end
						annotList:Add(menuItem)
					end
				end
			end
		end
		if (200-Y)<delta and Annotations[region][annotRow+1]~=nil then
			if Annotations[region][annotRow+1][annotCol+1]~=nil then
				local Y2=Y-200
				for k,v in ipairs(Annotations[region][annotRow+1][annotCol+1]) do
					if math.abs(X2-v.X)<delta and math.abs(Y2-v.Y)<delta then
						local menuItem
						local name=v.Name
						if name=="" then name=v.Description end
						if annotIcon[v.Type]==nil then
							menuItem=Turbine.UI.MenuItem(name.." (...)")
						else
							menuItem=Turbine.UI.MenuItem(name.." ("..Resource[language][annotIcon[v.Type].StringID]..")")
						end
						menuItem.Name=name
						menuItem.Description=v.Description
						menuItem.ID=v.id
						menuItem.Click=function(sender,args)
						-- show details
							if mapWindow.PopUp~=nil then
								mapWindow.PopUp:SetVisible(false)
								mapWindow.PopUp=nil
							end
							mapWindow.PopUp=PopUpDialog(sender.Name,sender.Description,4,"OK")
							mapWindow.PopUp:SetZOrder(mapWindow:GetZOrder()+1)
						end
						annotList:Add(menuItem)
					end
				end
			end
		end
	end
	
-- if x<delta and annotCol>0 then do same for prior column
-- if y<delta and annotRow>0 then do same for prior row
-- if x>(200-delta) and annotCol<gridmax then do same for next column
-- if y>(200-delta) and annotRow<gridmax then do same for next row
-- etc.
	local x,y=Turbine.UI.Display:GetMousePosition()
	annotHover:ShowMenuAt(x+2,y+15);
end


annotFilter=Turbine.UI.ContextMenu()
local filterList=annotFilter:GetItems()
local menuItem=Turbine.UI.MenuItem(Resource[language][30])
menuItem.ID=-1
menuItem.Click=function(sender,args)
	local items=annotFilter:GetItems()
	for k=3,items:GetCount() do
		items:Get(k):SetChecked(true)
		Settings.Filter[items:Get(k).ID]=true
	end
	mapWindow:RefreshAnnotations()
end
filterList:Add(menuItem)
local menuItem=Turbine.UI.MenuItem(Resource[language][31])
menuItem.ID=-1
menuItem.Click=function(sender,args)
	local items=annotFilter:GetItems()
	for k=3,items:GetCount() do
		items:Get(k):SetChecked(false)
		Settings.Filter[items:Get(k).ID]=false
	end
	mapWindow:RefreshAnnotations()
end
filterList:Add(menuItem)
local tmpTable={}
for k,v in pairs(annotIcon) do
	table.insert(tmpTable,{["ID"]=k,["String"]=Resource[language][v.StringID]})
end
table.sort(tmpTable,function(args1,args2)if args1.String<args2.String then return true end end)
for k,v in ipairs(tmpTable) do
	local menuItem=Turbine.UI.MenuItem(v.String)
	menuItem.ID=v.ID
	menuItem:SetChecked(Settings.Filter[v.ID])
	menuItem.Click=function(sender,args)
		sender:SetChecked(not sender:IsChecked())
		Settings.Filter[sender.ID]=sender:IsChecked()
		mapWindow:RefreshAnnotations()
	end
	filterList:Add(menuItem)
end
tmpTable=nil
mapWindow = Turbine.UI.Window();
mapWindow:SetOpacity(Settings.Opacity)
mapWindow:SetWantsKeyEvents(true);
mapWindow.ZoomLevel=1
mapWindow.AnnotLoader=Turbine.UI.Control()

mapWindow:SetSize(displayWidth*Settings.WidthPercent/100,displayHeight*Settings.HeightPercent/100);
mapWindow:SetPosition(displayWidth*Settings.LeftPercent/100,displayHeight*Settings.TopPercent/100);
mapWindow:SetBackColor(Turbine.UI.Color(0,0,0));
mapWindow.X=1;
mapWindow.Y=1;
mapWindow.Moving=false

mapWindow.MoveTab=Turbine.UI.Control()
mapWindow.MoveTab:SetParent(mapWindow)
mapWindow.MoveTab:SetSize(20,20)
mapWindow.MoveTab:SetBackground(resourcePath.."Move.jpg")
mapWindow.MoveTab.MouseDown=function(sender,args)
	mapWindow.Moving=true
	mapWindow.MoveX=args.X
	mapWindow.MoveY=args.Y
end
mapWindow.MoveTab.MouseUp=function(sender,args)
	mapWindow.Moving=false
end
mapWindow.MoveTab.MouseMove=function(sender,args)
	if mapWindow.Moving then
		local left=mapWindow:GetLeft()+args.X-mapWindow.MoveX
		local top=mapWindow:GetTop()+args.Y-mapWindow.MoveY
		if left<0 then left=0 end
		if top<0 then top=0 end
		if left>displayWidth-10 then left=displayWidth-10 end
		if top>displayHeight-10 then top=displayHeight-10 end
		mapWindow:SetPosition(left,top)
	end
end

mapWindow.OptionsButton=Turbine.UI.Control()
mapWindow.OptionsButton:SetParent(mapWindow)
mapWindow.OptionsButton:SetSize(20,20)
mapWindow.OptionsButton:SetPosition(30,0)
mapWindow.OptionsButton:SetBackground(resourcePath.."options.tga")
mapWindow.OptionsButton.MouseEnter=function()
	mapWindow.OptionsButton:SetBackground(resourcePath.."options_hover.tga")
end
mapWindow.OptionsButton.MouseLeave=function()
	mapWindow.OptionsButton:SetBackground(resourcePath.."options.tga")
end
mapWindow.OptionsButton.MouseClick=function()
	RefreshOptionValues()
	optionsWindow:SetVisible(true)
end

mapWindow.Region=RadioButtonGroup()
mapWindow.Region:SetParent(mapWindow)
mapWindow.Region:SetSize(300,20)
mapWindow.Region:SetPosition(60,0)
mapWindow.Region.UnselectedIcon=resourcePath.."RB_unselected.tga";
mapWindow.Region.SelectedIcon=resourcePath.."RB_selected.tga";
mapWindow.Region.IconWidth=16;
mapWindow.Region.IconHeight=16;
mapWindow.Region:AddChoice(Resource[language][13]);
mapWindow.Region:AddChoice(Resource[language][14]);
mapWindow.Region:AddChoice(Resource[language][24]);
mapWindow.Region:AddChoice(Resource[language][56]);
mapWindow.Region:SetRows(1);
mapWindow.Region:SetSelectedChoice(1);
mapWindow.Region.SelectionChanged=function()
	mapWindow.DoNotReload=true
	local region=mapWindow.Region:GetSelectedChoice()
	local zoom=mapWindow.Zoom:GetValue()
	if mapWindow.Zoom:GetSelectedChoice()/2==math.floor(mapWindow.Zoom:GetSelectedChoice()/2) then zoom=zoom/2 end
	local maxRows=math.floor(((MaxRow[region]-1)/zoom)-mapWindow.GridRows+3)*200-1
	local maxCols=math.floor(((MaxCol[region]-1)/zoom)-mapWindow.GridCols+3)*200-1

	if mapWindow.vGridScroll:GetValue()>maxRows then
		mapWindow.vGridScroll:SetValue(maxRows)
	end
	if mapWindow.hGridScroll:GetValue()>maxCols then
		mapWindow.hGridScroll:SetValue(maxCols)
	end
	mapWindow.vGridScroll:SetMaximum(maxRows)
	mapWindow.hGridScroll:SetMaximum(maxCols)
	mapWindow.DoNotReload=false
	mapWindow.ReloadGrid()
end

mapWindow.NSCap=Turbine.UI.Label()
mapWindow.NSCap:SetParent(mapWindow)
mapWindow.NSCap:SetBackColor(Turbine.UI.Color.Black)
mapWindow.NSCap:SetSize(40,20)
mapWindow.NSCap:SetPosition(380,0)
mapWindow.NSCap:SetFont(UIFont);
mapWindow.NSCap:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
mapWindow.NSCap:SetText(Resource[language][20].."/"..Resource[language][21]..": ")

mapWindow.NS=Turbine.UI.TextBox()
mapWindow.NS:SetParent(mapWindow)
mapWindow.NS:SetBackColor(Turbine.UI.Color.Black)
mapWindow.NS:SetSize(70,20)
mapWindow.NS:SetPosition(420,0)
mapWindow.NS:SetFont(UIFont);
mapWindow.NS:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft);
mapWindow.NS:SetText("")

mapWindow.EWCap=Turbine.UI.Label()
mapWindow.EWCap:SetParent(mapWindow)
mapWindow.EWCap:SetBackColor(Turbine.UI.Color.Black)
mapWindow.EWCap:SetSize(40,20)
mapWindow.EWCap:SetPosition(490,0)
mapWindow.EWCap:SetFont(UIFont);
mapWindow.EWCap:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleRight);
mapWindow.EWCap:SetText(Resource[language][22].."/"..Resource[language][23]..": ")

mapWindow.EW=Turbine.UI.TextBox()
mapWindow.EW:SetParent(mapWindow)
mapWindow.EW:SetBackColor(Turbine.UI.Color.Black)
mapWindow.EW:SetSize(70,20)
mapWindow.EW:SetPosition(530,0)
mapWindow.EW:SetFont(UIFont);
mapWindow.EW:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft);
mapWindow.EW:SetText("")
mapWindow.DoNotReload=false

mapWindow.Zoom=RadioButtonGroup()
mapWindow.Zoom:SetParent(mapWindow)
mapWindow.Zoom:SetSize(300,20)
mapWindow.Zoom:SetPosition(600,0)
mapWindow.Zoom.UnselectedIcon=resourcePath.."RB_unselected.tga";
mapWindow.Zoom.SelectedIcon=resourcePath.."RB_selected.tga";
mapWindow.Zoom.IconWidth=16;
mapWindow.Zoom.IconHeight=16;
mapWindow.Zoom:AddChoice("x1",1);
mapWindow.Zoom:AddChoice("x2",2);
mapWindow.Zoom:AddChoice("x4",4);
mapWindow.Zoom:AddChoice("x8",8);
mapWindow.Zoom:AddChoice("x16",16);
mapWindow.Zoom:AddChoice("x32",32);
mapWindow.Zoom:SetRows(1);
mapWindow.Zoom:SetSelectedChoice(1);
mapWindow.Zoom.SelectionChanged=function()
--mapWindow.hGridScroll:SetMaximum((MaxCol[1]-mapWindow.GridCols+2)*200-1)
	mapWindow.DoNotReload=true
	local region=mapWindow.Region:GetSelectedChoice()
	local zoom=mapWindow.Zoom:GetValue()
	if mapWindow.Zoom:GetSelectedChoice()/2==math.floor(mapWindow.Zoom:GetSelectedChoice()/2) then zoom=zoom/2 end
	local maxRows=math.floor(((MaxRow[region]-1)/zoom)-mapWindow.GridRows+3)*200-1
	local maxCols=math.floor(((MaxCol[region]-1)/zoom)-mapWindow.GridCols+3)*200-1

	local oldLeft=mapWindow.mapGrid:GetLeft()
	local oldTop=mapWindow.mapGrid:GetTop()
	local oldVScrollVal=mapWindow.vGridScroll:GetValue()
	local oldHScrollVal=mapWindow.hGridScroll:GetValue()
	local vScrollVal=oldVScrollVal/mapWindow.vGridScroll:GetMaximum()*maxRows
	local hScrollVal=oldHScrollVal/mapWindow.hGridScroll:GetMaximum()*maxCols

	local oldZoom=2^(mapWindow.ZoomLevel-1)
	local oldZoomLevel=mapWindow.ZoomLevel
	if oldZoomLevel/2==math.floor(oldZoomLevel/2) then
		oldZoom=oldZoom/2
	end
	local x=(oldHScrollVal/200*oldZoom+mapWindow.mapViewPort:GetWidth()/400*oldZoom+1)*.8
	local y=(oldVScrollVal/200*oldZoom+mapWindow.mapViewPort:GetHeight()/400*oldZoom+1)*.8

	local ns=Grid[region].NSOffset-y
	local ew=Grid[region].EWOffset+x

	if mapWindow.vGridScroll:GetValue()>maxRows then
		mapWindow.vGridScroll:SetValue(maxRows)
	end
	if mapWindow.hGridScroll:GetValue()>maxCols then
		mapWindow.hGridScroll:SetValue(maxCols)
	end
	mapWindow.vGridScroll:SetMaximum(maxRows)
	mapWindow.hGridScroll:SetMaximum(maxCols)
	mapWindow.vGridScroll:SetValue(vScrollVal)
	mapWindow.hGridScroll:SetValue(hScrollVal)

	mapWindow.ZoomLevel=mapWindow.Zoom:GetSelectedChoice()

	mapWindow.MakeGrid()
	mapWindow.DoNotReload=false
	mapWindow.ShowLocation(region,ns,ew)
end

mapWindow.CenterQS=Turbine.UI.Lotro.Quickslot()
mapWindow.CenterQS:SetParent(mapWindow)
mapWindow.CenterQS:SetSize(26,20)
mapWindow.CenterQS:SetPosition(900,0)
mapWindow.CenterQS:SetAllowDrop(false);

mapWindow.Center=Turbine.UI.Control()
mapWindow.Center:SetParent(mapWindow)
mapWindow.Center:SetSize(26,20)
mapWindow.Center:SetPosition(900,0)
mapWindow.Center:SetBackground(resourcePath.."CenterButton.jpg")
mapWindow.Center:SetMouseVisible(false)

mapWindow.Center.Cmd="/loc"
locale=Turbine.Engine:GetLanguage()
if locale==Turbine.Language.French then
	mapWindow.Center.Cmd="/emp"
elseif locale==Turbine.Language.German then
	mapWindow.Center.Cmd="/pos"
end
mapWindow.CenterQS:SetShortcut(Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Alias, mapWindow.Center.Cmd));

mapWindow.CenterQS.DragDrop=function()
	-- reestablish the shortcut if the user accidentally drags it onto itself which wipes it out
	mapWindow.CenterQS:SetShortcut(Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Alias, mapWindow.Center.Cmd));
end

mapWindow.Usage=Turbine.UI.Control()
mapWindow.Usage:SetParent(mapWindow)
mapWindow.Usage:SetSize(26,20)
mapWindow.Usage:SetPosition(926,0)
mapWindow.Usage:SetBackground(resourcePath.."UsageButton.jpg")
mapWindow.Usage.MouseClick=function()
	usageWindow.UpdateUsage()
	usageWindow:SetVisible(true)
end

mapWindow.Search=Turbine.UI.Control()
mapWindow.Search:SetParent(mapWindow)
mapWindow.Search:SetSize(26,20)
mapWindow.Search:SetPosition(952,0)
mapWindow.Search:SetBackground(resourcePath.."SearchButton.jpg")
mapWindow.Search.MouseClick=function()
	searchWindow:SetVisible(true)
	searchText:Focus()
end

mapWindow.CloseButton=Turbine.UI.Control()
mapWindow.CloseButton:SetParent(mapWindow)
mapWindow.CloseButton:SetSize(16,16)
mapWindow.CloseButton:SetPosition(mapWindow:GetWidth()-17,1)
mapWindow.CloseButton:SetBackground(0x41000196)
mapWindow.CloseButton.MouseClick=function()
	mapWindow:SetVisible(false)
	icon:SetVisible(true)
end
mapWindow.CloseButton.MouseEnter=function()
	mapWindow.CloseButton:SetBackground(0x41000198)
end
mapWindow.CloseButton.MouseLeave=function()
	mapWindow.CloseButton:SetBackground(0x41000196)
end

-- we need to create a grid of 200x200 controls that get backgrounds assigned from the list of tiles

mapWindow.mapViewPort=Turbine.UI.Control();
mapWindow.mapViewPort:SetParent(mapWindow);
mapWindow.mapViewPort:SetSize(mapWindow:GetWidth()-10,mapWindow:GetHeight()-30);
mapWindow.mapViewPort:SetZOrder(2);
mapWindow.mapViewPort:SetPosition(0,20);
mapWindow.mapViewPort:SetBackColor(Turbine.UI.Color(0,0,0));

mapWindow.Hourglass=Turbine.UI.Control()
mapWindow.Hourglass:SetParent(mapWindow.mapViewPort)
mapWindow.Hourglass:SetSize(32,32)
mapWindow.Hourglass:SetVisible(false)
mapWindow.Hourglass:SetPosition(mapWindow:GetWidth()/2-16,mapWindow:GetHeight()/2-16)
mapWindow.Hourglass:SetBlendMode(Turbine.UI.BlendMode.Overlay)
mapWindow.Hourglass:SetBackground(resourcePath.."hourglass.tga")
mapWindow.Hourglass:SetZOrder(3)

mapWindow.GridRows=math.floor(mapWindow.mapViewPort:GetHeight()/200)+2
mapWindow.GridCols=math.floor(mapWindow.mapViewPort:GetWidth()/200)+2

mapWindow.mapGrid=Turbine.UI.Control();
mapWindow.mapGrid:SetParent(mapWindow.mapViewPort);
mapWindow.mapGrid:SetSize(mapWindow.GridCols*200,mapWindow.GridRows*200);
mapWindow.mapGrid:SetPosition(-200,-200);
mapWindow.mapGrid:SetBackColor(Turbine.UI.Color(0,0,0));

mapWindow.PlayerPos=nil
mapWindow.PlayerIcon=Turbine.UI.Control()
mapWindow.PlayerIcon:SetParent(mapWindow.mapGrid)
mapWindow.PlayerIcon:SetBackground(resourcePath.."Player20.tga")
mapWindow.PlayerIcon.image1=resourcePath.."Player20.tga"
mapWindow.PlayerIcon.image2=resourcePath.."Player40.tga"
mapWindow.PlayerIcon:SetSize(20,20)
mapWindow.PlayerIcon:SetMouseVisible(false)
mapWindow.PlayerIcon:SetBlendMode(Turbine.UI.BlendMode.Overlay)
mapWindow.PlayerIcon:SetZOrder(3)

-- NSOffset is calculated from the +/- NS location at the corner of the map
-- YOffSet is 200*number of tiles off set

Grid[1].NSOffset=26
Grid[1].EWOffset=-110.75

Grid[2].NSOffset=65.22
--Grid[2].NSOffset=1.22
Grid[2].EWOffset=-119.6

Grid[3].NSOffset=-15.6
Grid[3].EWOffset=-96.4

Grid[4].NSOffset=-15.6
--Grid[4].EWOffset=-96.4
Grid[4].EWOffset=-25.2

Grid[1].YOffSet=37400
Grid[1].XOffSet=9200
Grid[2].YOffSet=47200
--Grid[2].YOffSet=31200
Grid[2].XOffSet=7000
Grid[3].YOffSet=21200
Grid[3].XOffSet=12800
--Grid[3].YOffSet=25800
Grid[3].YOffSet=27000
Grid[3].XOffSet=12800

Grid[4].YOffSet=27000
--Grid[4].XOffSet=5000
Grid[4].XOffSet=30600

mapWindow.Ping=Turbine.UI.Control();
mapWindow.Ping:SetParent(mapWindow.mapGrid)
mapWindow.Ping:SetPosition(-200,-200)
mapWindow.Ping:SetZOrder(4)
mapWindow.Ping.State=0;
mapWindow.Ping:SetVisible(true);
mapWindow.Ping:SetSize(23,23);
mapWindow.Ping:SetWantsUpdates(false);
mapWindow.Ping.Counter=1;
mapWindow.Ping.IconBase=resourcePath.."Ping"
mapWindow.Ping.MaxCounter=2;
mapWindow.Ping:SetBlendMode(Turbine.UI.BlendMode.Overlay)
mapWindow.Ping:SetBackground(mapWindow.Ping.IconBase.."1.tga");
mapWindow.Ping.Region=1
mapWindow.Ping.Row=3
mapWindow.Ping.Col=3
mapWindow.Ping.MouseClick=function(sender,args)
	mapWindow.Ping.State=0; -- it will auto hide itmapWindow on next tick
end
mapWindow.Ping.Update=function(sender)
	if mapWindow.Ping.State==0 then
		mapWindow.Ping:SetWantsUpdates(false);
		mapWindow.Ping:SetPosition(-200,-200);
		mapWindow.Ping.Counter=0;
		mapWindow.Ping:SetBackground(mapWindow.Ping.IconBase.."1.tga");
	elseif mapWindow.Ping.State==7 then
		mapWindow.Ping.Counter=mapWindow.Ping.Counter+1;
		if mapWindow.Ping.Counter>mapWindow.Ping.MaxCounter then
			mapWindow.Ping.Counter=0;
			mapWindow.Ping.State=3;
			mapWindow.Ping:SetBackground(mapWindow.Ping.IconBase.."3.tga");
		end

	else
		mapWindow.Ping.Counter=mapWindow.Ping.Counter+1;
		if mapWindow.Ping.Counter>mapWindow.Ping.MaxCounter then
			mapWindow.Ping.Counter=0;
			mapWindow.Ping.State=mapWindow.Ping.State+1;
			mapWindow.Ping:SetBackground(mapWindow.Ping.IconBase..tostring(mapWindow.Ping.State)..".tga");
		end
	end
end

mapWindow.GridCells={};
mapWindow.GridSelectedResource=""
mapWindow.MakeGrid=function()
	-- remove prior grid, especially event handlers,if any
	local region=mapWindow.Region:GetSelectedChoice()
	for row,rowData in ipairs(mapWindow.GridCells) do
		for col,colData in ipairs(rowData) do
			mapWindow.GridCells[row][col].MouseEnter=nil
			mapWindow.GridCells[row][col].MouseLeave=nil
			mapWindow.GridCells[row][col].MouseUp=nil
			mapWindow.GridCells[row][col].MouseDown=nil
			mapWindow.GridCells[row][col].MouseClick=nil
			mapWindow.GridCells[row][col].MouseMove=nil
			mapWindow.GridCells[row][col].ResourceID=nil
			mapWindow.GridCells[row][col].SelectedOverlay=nil
			mapWindow.GridCells[row][col]=nil
		end
		mapWindow.GridCells[row]=nil		
	end

	-- create new grid
	mapWindow.mapViewPort:SetStretchMode(0)
	local zoomMult=1
	-- zoomlevel = 1,2,3,4
	if mapWindow.ZoomLevel/2==math.floor(mapWindow.ZoomLevel/2) then
		zoomMult=2
		mapWindow.Ping.IconBase=resourcePath.."Ping2_"
		mapWindow.Ping:SetSize(46,46);
	else
		mapWindow.Ping.IconBase=resourcePath.."Ping"
		mapWindow.Ping:SetSize(23,23);
	end

	mapWindow.mapViewPort:SetSize((mapWindow:GetWidth()-10)*zoomMult,(mapWindow:GetHeight()-30)*zoomMult);
	mapWindow.GridRows=math.floor((mapWindow:GetHeight()-30)/200)+2
	mapWindow.GridCols=math.floor((mapWindow:GetWidth()-10)/200)+2
	mapWindow.mapGrid:SetSize(mapWindow.GridCols*200*zoomMult,mapWindow.GridRows*200*zoomMult);

	for row=1,mapWindow.GridRows*zoomMult do
		mapWindow.GridCells[row]={}
		for col=1,mapWindow.GridCols*zoomMult do
			mapWindow.GridCells[row][col]=Turbine.UI.Label()
			mapWindow.GridCells[row][col]:SetParent(mapWindow.mapGrid)
			mapWindow.GridCells[row][col]:SetSize(200,200)
			mapWindow.GridCells[row][col]:SetPosition((col-1)*200,(row-1)*200)
			mapWindow.GridCells[row][col].Row=row
			mapWindow.GridCells[row][col].Col=col

			mapWindow.GridCells[row][col].MouseDown=function(sender,args)
				mapWindow.Panning=true
				mapWindow.PanX,mapWindow.PanY=Turbine.UI.Display:GetMousePosition()
				mapWindow.ScrollStartX=mapWindow.hGridScroll:GetValue()
				mapWindow.ScrollStartY=mapWindow.vGridScroll:GetValue()
			end
			mapWindow.GridCells[row][col].MouseUp=function(sender,args)
				mapWindow.Panning=false
				mapWindow.ReloadGrid()
			end
			mapWindow.GridCells[row][col].MouseDoubleClick=function(sender,args)
				local zoom=mapWindow.Zoom:GetValue()
				if mapWindow.Zoom:GetSelectedChoice()/2==math.floor(mapWindow.Zoom:GetSelectedChoice()/2) then zoom=zoom/2 end
					
				local X=((mapWindow.X+sender.Col-2)*zoom+1+args.X/200*zoom)*.8
				local Y=((mapWindow.Y+sender.Row-2)*zoom+1+args.Y/200*zoom)*.8

				local region=mapWindow.Region:GetSelectedChoice()
				local NS=math.floor((Grid[region].NSOffset-Y)*100)/100
				local EW=math.floor((Grid[region].EWOffset+X)*100)/100
				local zoomLevel=mapWindow.Zoom:GetSelectedChoice()
				if args.Button==1 then
					if zoomLevel>1 then
						mapWindow.Zoom:SetSelectedChoice(zoomLevel-1)
					end
				else
					if zoomLevel<6 then
						mapWindow.Zoom:SetSelectedChoice(zoomLevel+1)
					end
				end
				mapWindow.ShowLocation(region,NS,EW)
			end
			mapWindow.GridCells[row][col].MouseClick=function(sender,args)
				local zoom=mapWindow.Zoom:GetValue()
				if args.Button==2 and Settings.ShowAnnotations and zoom<3 then
					-- popup filter menu
					local x,y=Turbine.UI.Display:GetMousePosition()
					annotFilter:ShowMenuAt(x+2,y+15);	
				end
--				Turbine.Shell.WriteLine("Row:"..tostring(sender.Row+mapWindow.Y)..", Col:"..tostring(sender.Col+mapWindow.X))
--				Turbine.Shell.WriteLine("Y:"..tostring(args.Y)..", X:"..tostring(args.X))
			end
			mapWindow.GridCells[row][col].MouseMove=function(sender,args)
				if mapWindow.Panning then
					local panX,panY=Turbine.UI.Display:GetMousePosition()
					local newVal=mapWindow.ScrollStartX+(mapWindow.PanX-panX)*zoomMult
					if newVal<0 then newVal=0 end
					mapWindow.hGridScroll:SetValue(newVal)
					newVal=mapWindow.ScrollStartY+(mapWindow.PanY-panY)*zoomMult
					if newVal<0 then newVal=0 end
					mapWindow.vGridScroll:SetValue(newVal)
				else
					local zoom=mapWindow.Zoom:GetValue()
					if mapWindow.Zoom:GetSelectedChoice()/2==math.floor(mapWindow.Zoom:GetSelectedChoice()/2) then zoom=zoom/2 end
					
					local X=((mapWindow.X+sender.Col-2)*zoom+1+args.X/200*zoom)*.8
					local Y=((mapWindow.Y+sender.Row-2)*zoom+1+args.Y/200*zoom)*.8

					local region=mapWindow.Region:GetSelectedChoice()
					local NS=math.floor((Grid[region].NSOffset-Y)*100)/100
					local EW=math.floor((Grid[region].EWOffset+X)*100)/100
					if NS<0 then
						mapWindow.NS:SetText(tostring(0-NS).." "..Resource[language][21])
					else
						mapWindow.NS:SetText(tostring(NS).." "..Resource[language][20])
					end
					if EW<0 then
						mapWindow.EW:SetText(tostring(0-EW).." "..Resource[language][23])
					else
						mapWindow.EW:SetText(tostring(EW).." "..Resource[language][22])
					end
				end
			end
		end
	end
	if zoomMult==2 then
		mapWindow.mapViewPort:SetStretchMode(1)
		mapWindow.mapViewPort:SetSize(mapWindow:GetWidth()-10,mapWindow:GetHeight()-30);
	end
end
mapWindow.MakeGrid()
mapWindow.ClearAnnotations=function()
-- really need to clear all but search results...
	Annotations={}
end
mapWindow.Annotations={}
mapWindow.ReloadGrid=function()
	if mapWindow.DoNotReload then return end -- cheesy but effective way to use flag to prevent excessive scroll & reload events from resizing, etc

	local debug=(debugWindow~=nil and debugWindow:IsVisible())
	local region=mapWindow.Region:GetSelectedChoice()
	local x=math.floor(mapWindow.hGridScroll:GetValue()/200)
	local y=math.floor(mapWindow.vGridScroll:GetValue()/200)
	local largeIcon=(mapWindow.ZoomLevel/2==math.floor(mapWindow.ZoomLevel/2))
	local zoom=mapWindow.Zoom:GetValue()
	if largeIcon then zoom=zoom/2 end
	if x==nil then x=0 end
	if y==nil then y=0 end
	if mapWindow.ZoomLevel>2 then
		x=x*zoom
		y=y*zoom
	end
	-- if zoom>2 then clear annotations
	if mapWindow.ZoomLevel>2 then
		-- we only display/cache annotations at zoom 1 or 2
		mapWindow.ClearAnnotations()
	else
		if Settings.ShowAnnotations then
			if largeIcon then
				mapWindow.Hourglass:SetPosition(mapWindow:GetWidth()-32,mapWindow:GetHeight()-32)
				mapWindow.Hourglass:SetSize(64,64)
				mapWindow.Hourglass:SetBackground(resourcePath.."hourglass64.tga")
			else
				mapWindow.Hourglass:SetPosition(mapWindow:GetWidth()/2-16,mapWindow:GetHeight()/2-32)
				mapWindow.Hourglass:SetSize(32,32)
				mapWindow.Hourglass:SetBackground(resourcePath.."hourglass.tga")
			end
			mapWindow.Hourglass:SetVisible(true)
		end
	end
	mapWindow.PlayerIcon:SetVisible(false)
	local minH=math.floor(x/4)*4+1
	local minV=math.floor(y/4)*4+1
	local maxV=minV+math.ceil(#mapWindow.GridCells/4)*4-1 -- num rows
	local maxH=minH+math.ceil(#mapWindow.GridCells[1]/4)*4-1 -- num cols
	-- clear displayed annotations to make scrolling a little less, umm, odd
	if mapWindow.Annotations~=nil then
		for k,v in pairs(mapWindow.Annotations) do
			v.MouseClick=nil
			v.MouseHover=nil
			v:SetVisible(false)
			mapWindow.Annotations[k]=nil
		end
	end
	-- remove annotations that are no longer on grid
	for k,v in pairs(Annotations) do
		if k==region then
			for row,Cols in pairs(v) do
				if row<minV or row>maxV then
					Annotations[k][row]=nil
				else
					for col,Cell in pairs(Cols) do
						if col<minH or col>maxH or not Settings.ShowAnnotations then
							Annotations[k][row][col]=nil
						end
					end
				end
			end
		else
			Annotations[k]=nil
		end
	end
	-- clear any previous un-processed annotation requests
	for k,v in ipairs(mapWindow.AnnotCmds) do
		Turbine.Shell.RemoveCommand(v)
		mapWindow.AnnotCmds[k]=nil
	end

--	if Annotations[region]==nil then Annotations[region]={} end
-- caching is temporarilly disabled until I figure out the intermittant load failures
	Annotations[region]={}

	mapWindow.Ping:SetPosition(-200,-200)
	local pingV=math.floor((mapWindow.Ping.Row-1)/zoom)*zoom+1
	local pingH=math.floor((mapWindow.Ping.Col-1)/zoom)*zoom+1
	for row,rowCells in ipairs(mapWindow.GridCells) do
		for col,Cell in ipairs(rowCells) do
			local tmpRegion=Grid[region]
			local tmpCell
			local row2=(row-1)*zoom+y+1
			local col2=(col-1)*zoom+x+1
			if Annotations[region][row2]==nil then Annotations[region][row2]={} end
			if mapWindow.Ping.State>0 and mapWindow.Ping.Region==region and pingV==row2 and pingH==col2 then
				local pingX=mapWindow.Ping.X/zoom+(mapWindow.Ping.Col-pingH)*200/zoom
				local pingY=mapWindow.Ping.Y/zoom+(mapWindow.Ping.Row-pingV)*200/zoom
				if largeIcon then
					mapWindow.Ping:SetPosition(Cell:GetLeft()+pingX-23,Cell:GetTop()+pingY-23)
				else
					mapWindow.Ping:SetPosition(Cell:GetLeft()+pingX-11,Cell:GetTop()+pingY-11)
				end
			end
			if mapWindow.ZoomLevel<3 and Annotations[region][row2][col2]==nil and Settings.ShowAnnotations then
				local row3, col3
				row3=math.floor((row2-1)/4)*4+1
				col3=math.floor((col2-1)/4)*4+1
				if Annotations[region][row3]==nil then Annotations[region][row3]={} end
				if Annotations[region][row3][col3]==nil then
					Annotations[region][row3][col3]={} -- flags as cached
					local cmd="0TMCL_"..tostring(region).."_"..tostring(row3).."_"..tostring(col3)
					if not Turbine.Shell.IsCommand(cmd) then
						local tmpCmd=Turbine.ShellCommand()
						table.insert(mapWindow.AnnotCmds,tmpCmd)
						Turbine.Shell.AddCommand(cmd,tmpCmd)
					end
				end
--				Annotations[region][row2][col2]={} -- not sure this is even needed anymore
			end
			if tmpRegion~=nil then
				local tmpRow=tmpRegion[row2]
				if tmpRow==nil then tmpRow={} end
				local tmpCol=tmpRow[col2]
				if tmpCol==nil then tmpCol={} end
				if zoom==1 then
					tmpCell=tmpCol[zoom]
				else
					tmpCell=tostring(region).."_"..tostring(row2).."_"..tostring(col2).."_"..tostring(zoom)..".jpg"
				end
			end
			if tmpCell==nil then
				Cell:SetBackColor(Turbine.UI.Color(.2,.2,.2))
				Cell.ResourceID=nil
			else
				Cell.ResourceID=tmpCell
				Cell:SetBackColor(nil)
			end
			if debug then
				if tmpCell==nil or type(tmpCell)=="string" then
					Cell:SetText("x:"..tostring(col2).." y:"..tostring(row2).."\n"..tostring(tmpCell).."\n"..tilePath..tostring(tmpCell))
				else
					Cell:SetText("x:"..tostring(col2).." y:"..tostring(row2).."\n"..tmpCell.."\n"..string.format("0x%x",tonumber(tmpCell)))
				end
			else
				Cell:SetText("")
			end
			if type(tmpCell)=="string" then
				if tmpCell~=nil then
					local success,result=pcall(Turbine.UI.Control.SetBackground,Cell,tilePath..tostring(tmpCell))
					if not success then
						Cell:SetBackground(nil)
						Cell:SetBackColor(Turbine.UI.Color(.2,.2,.2))
					end
				end
			else
				Cell:SetBackground(tmpCell)
			end
		end
	end
	if mapWindow.PlayerPos~=nil and mapWindow.PlayerPos.region==region then
		if largeIcon then
			mapWindow.PlayerIcon:SetSize(40,40)
			mapWindow.PlayerIcon:SetBackground(mapWindow.PlayerIcon.image2)
			mapWindow.PlayerIcon:SetPosition(((mapWindow.PlayerPos.gridH-1)/zoom-mapWindow.X+1)*200+mapWindow.PlayerPos.gridX/zoom-20,((mapWindow.PlayerPos.gridV-1)/zoom-mapWindow.Y+1)*200+mapWindow.PlayerPos.gridY/zoom-20)
		else
			mapWindow.PlayerIcon:SetSize(20,20)
			mapWindow.PlayerIcon:SetBackground(mapWindow.PlayerIcon.image1)
			mapWindow.PlayerIcon:SetPosition(((mapWindow.PlayerPos.gridH-1)/zoom-mapWindow.X+1)*200+mapWindow.PlayerPos.gridX/zoom-10,((mapWindow.PlayerPos.gridV-1)/zoom-mapWindow.Y+1)*200+mapWindow.PlayerPos.gridY/zoom-10)
		end

		mapWindow.PlayerIcon:SetVisible(true)
--		mapWindow.PlayerIcon:SetRotation({x=0,y=0,z=0-(mapWindow.PlayerPos.Heading-90)});
	end
--	if (not mapWindow.Panning) and mapWindow.ZoomLevel<3 and #mapWindow.AnnotCmds>0 and Settings.ShowAnnotations then
	if mapWindow.ZoomLevel<3 and #mapWindow.AnnotCmds>0 and Settings.ShowAnnotations then
		mapWindow.AnnotLoaderState=0
		mapWindow.AnnotLoader:SetWantsUpdates(true)
	else
		mapWindow.RefreshAnnotations() -- will clear any that shouldn't be showing...
	end
end
mapWindow.RefreshAnnotations=function()
	local region=mapWindow.Region:GetSelectedChoice()
	local x=math.floor(mapWindow.hGridScroll:GetValue()/200)
	local y=math.floor(mapWindow.vGridScroll:GetValue()/200)
	local largeIcon=(mapWindow.ZoomLevel/2==math.floor(mapWindow.ZoomLevel/2))
	local zoom=mapWindow.Zoom:GetValue()
	if largeIcon then zoom=zoom/2 end
	if x==nil then x=0 end
	if y==nil then y=0 end
	if mapWindow.ZoomLevel>2 then
		x=x*zoom
		y=y*zoom
	end
	if mapWindow.Annotations~=nil then
		for k,v in pairs(mapWindow.Annotations) do
			v.MouseClick=nil
			v.MouseHover=nil
			v:SetVisible(false)
			mapWindow.Annotations[k]=nil
		end
	end
	for row,rowCells in ipairs(mapWindow.GridCells) do
		for col,Cell in ipairs(rowCells) do
			local tmpRegion=Grid[region]
			local tmpCell
			local row2=(row-1)*zoom+y+1
			local col2=(col-1)*zoom+x+1
			local regAnnot=Annotations[region]
			if regAnnot~=nil then
				for annotRow=0,zoom-1 do
					local rowAnnot=regAnnot[row2+annotRow]
					if rowAnnot~=nil then
						for annotCol=0,zoom-1 do
							local colAnnot=rowAnnot[col2+annotCol]
							if colAnnot~=nil then
								for k,v in pairs(colAnnot) do
									local tmpIcon=Turbine.UI.Control()
									tmpIcon:SetParent(mapWindow.mapGrid)
									tmpIcon:SetBlendMode(Turbine.UI.BlendMode.Overlay)
									local annotType=v.Type
									if annotIcon[annotType]==nil then annotType=5 end
									if Settings.Filter[annotType]~=nil and (Settings.Filter[annotType]) then
										if largeIcon then
											tmpIcon:SetBackground(annotIcon[annotType].large)
											tmpIcon:SetSize(annotIcon[annotType].largeWidth,annotIcon[annotType].largeHeight)
											tmpIcon:SetPosition(Cell:GetLeft()+200*annotCol/zoom+v.X/zoom-annotIcon[annotType].largeWidth/2,Cell:GetTop()+200*annotRow/zoom+v.Y/zoom-annotIcon[annotType].largeHeight/2)
										else
											tmpIcon:SetBackground(annotIcon[annotType].small)
											tmpIcon:SetSize(annotIcon[annotType].smallWidth,annotIcon[annotType].smallHeight)
											tmpIcon:SetPosition(Cell:GetLeft()+200*annotCol/zoom+v.X/zoom-annotIcon[annotType].smallWidth/2,Cell:GetTop()+200*annotRow/zoom+v.Y/zoom-annotIcon[annotType].smallHeight/2)
										end
										tmpIcon.ID=k
										tmpIcon.X=v.X
										tmpIcon.Y=v.Y
										tmpIcon.Name=v.Name
										tmpIcon.Description=v.Description
										tmpIcon:SetZOrder(2)
										tmpIcon.MouseClick=function(sender,args)
											-- popup window with title and description
											if mapWindow.PopUp~=nil then
												mapWindow.PopUp:SetVisible(false)
												mapWindow.PopUp=nil
											end
											mapWindow.PopUp=PopUpDialog(sender.Name,sender.Description,4,"OK")
											mapWindow.PopUp:SetZOrder(mapWindow:GetZOrder()+1)
										end
										tmpIcon.MouseHover=function(sender,args)
											annotHover.Display(sender.ID,row2+annotRow,col2+annotCol,sender.X,sender.Y)
										end
										table.insert(mapWindow.Annotations,tmpIcon)
									end
								end
							end
						end
					end
				end
			end
		end
	end
	mapWindow.Hourglass:SetVisible(false)
end
-- main control needs scroll bars
mapWindow.vGridScroll=Turbine.UI.Lotro.ScrollBar();
mapWindow.vGridScroll:SetOrientation(Turbine.UI.Orientation.Vertical);
mapWindow.vGridScroll:SetParent(mapWindow);
mapWindow.vGridScroll:SetBackColor(Turbine.UI.Color(0,0,0));
mapWindow.vGridScroll:SetWidth(10);
mapWindow.vGridScroll:SetHeight(mapWindow.mapViewPort:GetHeight());
mapWindow.vGridScroll:SetPosition(mapWindow.mapViewPort:GetLeft()+mapWindow.mapViewPort:GetWidth(),mapWindow.mapViewPort:GetTop());
mapWindow.vGridScroll:SetMinimum(0)
mapWindow.vGridScroll:SetMaximum((MaxRow[1]-mapWindow.GridRows+2)*200-1)

mapWindow.vGridScroll:SetValue(0)
mapWindow.vGridScroll:SetVisible(true);
mapWindow.vGridScroll.ValueChanged=function(sender,args)
	local yValue=mapWindow.vGridScroll:GetValue();
	local y=math.floor(yValue/200)
	local yOffset=yValue-y*200

	mapWindow.mapGrid:SetTop(0-yOffset)

	y2=tonumber(mapWindow.Y)
	if (y+1)~=y2 then
		mapWindow.Y=y+1
		-- reload the grid images since we have a new start coordinate
		mapWindow.ReloadGrid()
	end
end

mapWindow.hGridScroll=Turbine.UI.Lotro.ScrollBar();
mapWindow.hGridScroll:SetOrientation(Turbine.UI.Orientation.Horizontal);
mapWindow.hGridScroll:SetParent(mapWindow);
mapWindow.hGridScroll:SetBackColor(Turbine.UI.Color(0,0,0));
mapWindow.hGridScroll:SetHeight(10);
mapWindow.hGridScroll:SetWidth(mapWindow.mapViewPort:GetWidth());
mapWindow.hGridScroll:SetPosition(mapWindow.mapViewPort:GetLeft(),mapWindow.mapViewPort:GetTop()+mapWindow.mapViewPort:GetHeight());
mapWindow.hGridScroll:SetMinimum(0)
mapWindow.hGridScroll:SetMaximum((MaxCol[1]-mapWindow.GridCols+2)*200-1)

mapWindow.hGridScroll:SetValue(0)
mapWindow.hGridScroll:SetZOrder(3);
mapWindow.hGridScroll:SetVisible(true);
mapWindow.hGridScroll.ValueChanged=function(sender,args)
	local xValue=mapWindow.hGridScroll:GetValue();
	local x=math.floor(xValue/200)

	local xOffset=xValue-x*200

	mapWindow.mapGrid:SetLeft(0-xOffset)

	x2=tonumber(mapWindow.X)
	if (x+1)~=x2 then
		mapWindow.X=x+1
		-- reload the grid images since we have a new start coordinate
		mapWindow.ReloadGrid()
	end
end

mapWindow.SizeTab=Turbine.UI.Control()
mapWindow.SizeTab:SetParent(mapWindow)
mapWindow.SizeTab:SetSize(10,10)
mapWindow.SizeTab:SetPosition(mapWindow:GetWidth()-10,mapWindow:GetHeight()-10)
mapWindow.SizeTab:SetBackground(resourcePath.."Resize.jpg")
mapWindow.SizeTab.MouseDown=function(sender,args)
	mapWindow.Sizing=true
	mapWindow.DoNotReload=true
	mapWindow.MoveX=args.X
	mapWindow.MoveY=args.Y
end
mapWindow.SizeTab.MouseUp=function(sender,args)
	mapWindow.Sizing=false
	-- need to recreate the grid since we just stretched it :(
	mapWindow.MakeGrid()
	mapWindow.vGridScroll.ValueChanged()
	mapWindow.hGridScroll.ValueChanged()
	mapWindow.DoNotReload=false
	mapWindow.ReloadGrid()
end
mapWindow.SizeTab.MouseMove=function(sender,args)
	if mapWindow.Sizing then
		local width=mapWindow:GetWidth()+args.X-mapWindow.MoveX
		local height=mapWindow:GetHeight()+args.Y-mapWindow.MoveY
		if width<30 then width=30 end
		if height<30 then height=30 end
		if width>displayWidth then width=displayWidth end
		if height>displayHeight then height=displayHeight end
		mapWindow:SetSize(width,height)
	end
end

hudVisible=true; -- terrible assumption but we have no choice
mapVisible=false;
usageVisible=false;
iconVisible=false;
optionsVisible=false;
searchVisible=false
mapWindow.KeyDown=function(sender, args)
--Turbine.Shell.WriteLine("KeyDown Action="..args.Action..", .Alt="..tostring(args.Alt)..", .Control="..tostring(args.Control)..", .Shift="..tostring(args.Shift))
	if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
		mapWindow:SetVisible(false)
		usageWindow:SetVisible(false)
		optionsWindow:SetVisible(false)
		searchWindow:SetVisible(false)
		if Settings.ShowIcon then
			icon:SetVisible(true)
		end
	elseif ( args.Action == 268435635 ) then -- toggle HUD
		hudVisible=not hudVisible;
		if hudVisible then
			mapWindow:SetVisible(mapVisible);
			usageWindow:SetVisible(usageVisible);
			icon:SetVisible(iconVisible);
			optionsWindow:SetVisible(optionsVisible);
			searchWindow:SetVisible(searchVisible)
		else
			usageVisible=usageWindow:IsVisible();
			usageWindow:SetVisible(false);
			mapVisible=mapWindow:IsVisible();
			mapWindow:SetVisible(false);
			iconVisible=icon:IsVisible();
			icon:SetVisible(false);
			optionsVisible=optionsWindow:IsVisible();
			optionsWindow:SetVisible(false);
			searchVisible=searchWindow:IsVisible();
			searchWindow:SetVisible(false);
		end
	end
end

plugin.Unload=function()
	Turbine.Shell.RemoveCommand(mapWindow.shellCmd)
	if mapWindow~=nil and mapWindow.chatListener~=nil then
		RemoveCallback(Turbine.Chat, "Received", mapWindow.chatListener);
		if mapWindow.AnnotCmds~=nil then
			for k,v in ipairs(mapWindow.AnnotCmds) do
				Turbine.Shell.RemoveCommand(v)
				mapWindow.AnnotCmds[k]=nil
			end
		end
	end
	-- if TMAnnot is loaded, unload it
	pcall(Turbine.PluginManager.UnloadScriptState,"TMAnnot")
	
	SaveData();
	-- remove shell commands
	Turbine.Shell.WriteLine("TerrainMap "..Plugins["TerrainMap"]:GetVersion().." by Garan unloaded");
	if searchCmd~=nil then
		Turbine.Shell.RemoveCommand(searchCmd)
	end
	-- unload the search plugin if it was currently loaded
	if searchWindow~=nil and searchWindow.SearchLoaded then
		Turbine.PluginManager.UnloadScriptState("TMSearch");
	end
end

mapWindow.chatListener=function(sender, args)
	-- check message for standard channel matching "You are at: r.* lx.* ly.* ox.* oy.* oz.* h.*"
	local locationString,headingString,patternString1,patternString2,patternString3,patternString4;
	locale=Turbine.Engine:GetLanguage()
	if locale==Turbine.Language.French then
		locationString="Votre emplacement .*: r.*"
		headingString="Votre emplacement .*: r.* h.*"
		patternString1="Votre emplacement est.*: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)"
		patternString2="Votre emplacement est.*: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)"
		patternString3="Votre emplacement est.*: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i(%d+) ox(%d+%.?%d*) oy(%d+%.?%d*) oz(%d+%.?%d*)"
		patternString4="Votre emplacement est.*: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i(%d+) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)"
	elseif locale==Turbine.Language.German then
		locationString="Ihr befindet Euch hier: r.*"
		headingString="Ihr befindet Euch hier: r.* h.*"
		patternString1="Ihr befindet Euch hier: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)"
		patternString2="Ihr befindet Euch hier: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)"
		patternString3="Ihr befindet Euch hier: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i(%d+) ox(%d+%.?%d*) oy(%d+%.?%d*) oz(%d+%.?%d*)"
		patternString4="Ihr befindet Euch hier: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i(%d+) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)"
	else
		-- russian is not currently supported by this plugin
		locationString="You are at: r.*"
		headingString="You are at: r.* h.*"
		patternString1="You are at: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)"
		patternString2="You are at: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)"
		patternString3="You are at: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i(%d+) ox(%d+%.?%d*) oy(%d+%.?%d*) oz(%d+%.?%d*)"
		patternString4="You are at: r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i(%d+) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)"
	end
	if args.ChatType==Turbine.ChatType.Standard and string.find(args.Message,locationString)~=nil then
		local region, lx, ly, ox, oy, oz, heading;
		if string.find(args.Message,headingString)==nil then
			-- there is a carriage return at the end of the message :(
			heading=0;
			region, lx, ly, ox, oy, oz = string.match(args.Message,patternString1);
			if region==nil then
				--You are at: r1 lx1984 ly1536 cInside ox105.19 oy-11.28 oz-56.47 h129.4
				region, lx, ly, ox, oy, oz, heading = string.match(args.Message,patternString2);
				if region==nil then
					region, lx, ly, i, ox, oy, oz, heading = string.match(args.Message,patternString3);
					if region==nil then
						region, lx, ly, i, ox, oy, oz, heading = string.match(args.Message,patternString4);
					end
				end
			end
		else
			region, lx, ly, ox, oy, oz, heading = string.match(args.Message,patternString1.." h(%d+%.?%d*)");
			if region==nil then
				--You are at: r1 lx1984 ly1536 cInside ox105.19 oy-11.28 oz-56.47 h129.4
				region, lx, ly, ox, oy, oz, heading = string.match(args.Message,patternString2.." h(%d+%.?%d*)");
				if region==nil then
					region, lx, ly, i, ox, oy, oz, heading = string.match(args.Message,patternString3.." h(%d+%.?%d*)");
					if region==nil then
						region, lx, ly, i, ox, oy, oz, heading = string.match(args.Message,patternString4.." h(%d+%.?%d*).*");
					end
				end
			end
		end
		if region ~= nil then
			if heading ==nil then
				-- when facing north the heading value is not present
				heading=0
			end
			region=tonumber(region);
			ox=euroNormalize(ox);
			oy=euroNormalize(oy);
			oz=euroNormalize(oz);
			heading=euroNormalize(heading);
			mapWindow.Region:SetSelectedChoice(region)
			local zoom=mapWindow.Zoom:GetValue()
			local zoomMult=1
			if mapWindow.Zoom:GetSelectedChoice()/2==math.floor(mapWindow.Zoom:GetSelectedChoice()/2) then
				zoom=zoom/2
				zoomMult=2
			end
			local xoffset,yoffset
			xoffset=(mapWindow:GetWidth()/2-5)*zoomMult
			yoffset=(mapWindow:GetHeight()/2-15)*zoomMult

			local h,v=(( math.floor( lx / 8 ) * 160 + ox ))*1.25-Grid[region].XOffSet,Grid[region].YOffSet-(( math.floor( ly / 8 ) * 160 + oy ))*1.25;

			mapWindow.PlayerPos={}
			mapWindow.PlayerPos.region=region
			mapWindow.PlayerPos.gridH=math.floor(h/200)/zoom
			mapWindow.PlayerPos.gridV=math.floor(v/200)/zoom
			mapWindow.PlayerPos.gridX=(h-mapWindow.PlayerPos.gridH*200)
			mapWindow.PlayerPos.gridY=(v-mapWindow.PlayerPos.gridV*200)
			mapWindow.PlayerPos.Heading=heading
			mapWindow.PlayerPos.gridH=mapWindow.PlayerPos.gridH+1
			mapWindow.PlayerPos.gridV=mapWindow.PlayerPos.gridV+1

			v=v/zoom-yoffset
			h=h/zoom-xoffset
			if v<0 then
				v=0
			end
			if h<0 then
				h=0
			end
			mapWindow.ReloadGrid()
			mapWindow.vGridScroll:SetValue(v)
			mapWindow.hGridScroll:SetValue(h)
		end
	end
end

AddCallback(Turbine.Chat, "Received", mapWindow.chatListener);
mapWindow.shellCmd=Turbine.ShellCommand();
Turbine.Shell.AddCommand("TerrainMap",mapWindow.shellCmd);

mapWindow.shellCmd.Execute = function(sender, cmd, args)
	if (string.lower(cmd) == "terrainmap") then
		local pos1=string.find(args," ",0,true);
		local command;
		args=string.lower(args)
		if pos1~=nil and pos1>0 then
			command=string.sub(args,1,pos1-1);
		else
			command=args;
		end
		if (command=="show") then
			mapWindow:SetVisible(true);
			icon:SetVisible(false)
		elseif (command=="hide") then
			mapWindow:SetVisible(false);
			usageWindow:SetVisible(false);
			optionsWindow:SetVisible(false);
			if Settings.ShowIcon then
				icon:SetVisible(true)
			end
		elseif (command=="toggle") then
			if mapWindow:IsVisible() then
				mapWindow:SetVisible(false);
				usageWindow:SetVisible(false);
				optionsWindow:SetVisible(false);
				if Settings.ShowIcon then
					icon:SetVisible(true)
				end
			else
				mapWindow:SetVisible(true);
				icon:SetVisible(false)
			end
		elseif (args=="debug on") then
			debugWindow:SetVisible(true);
		elseif (args=="debug off") then
			if debugWindow~=nil then
				debugWindow:SetVisible(false);
			end
		elseif (command=="add") and pos1~=nil then
			local argArray=string.split(string.sub(args,pos1+1),":");
			-- args are region:NS:EW:Title:Description
			-- where NS and EW are valid up to 2 decimal places
			local region=tonumber(argArray[1])
			if region~=nil and region>0 and region<=#Grid then
				argArray[2]=tonumber(argArray[2])
				argArray[3]=tonumber(argArray[3])
				if argArray[2]~=nil and argArray[3]~=nil then
					mapWindow.AddAnnotation(region, argArray[2], argArray[3], argArray[4], argArray[5], argArray[6])
				end
			end
		elseif (args=="usage") then
			usageWindow.UpdateUsage()
			usageWindow:SetVisible(true)
		else
			Turbine.Shell.WriteLine("usage:");
			Turbine.Shell.WriteLine("  /TerrainMap show");
			Turbine.Shell.WriteLine("       Displays the Terrain Map window.");
			Turbine.Shell.WriteLine("  /TerrainMap hide");
			Turbine.Shell.WriteLine("       Hides the Terrain Map window.");
			Turbine.Shell.WriteLine("  /TerrainMap toggle");
			Turbine.Shell.WriteLine("       Toggles the Terrain Map window (displays if it was hidden, hides if it was shown).");
			Turbine.Shell.WriteLine("  /TerrainMap debug on|off");
			Turbine.Shell.WriteLine("       Shows or Hides the Debug window.");
		end
	end
end
mapWindow.ShowLocation = function(region,ns,ew)
	mapWindow.Region:SetSelectedChoice(region)
	local zoom=mapWindow.Zoom:GetValue()
	local zoomMult=1
	if mapWindow.Zoom:GetSelectedChoice()/2==math.floor(mapWindow.Zoom:GetSelectedChoice()/2) then
		zoom=zoom/2
		zoomMult=2
	end
	-- now convert ns and ew to pixel coordinates values
--					local X=((mapWindow.X+sender.Col-2)*zoom+1+args.X/200*zoom)*.8
--					local Y=((mapWindow.Y+sender.Row-2)*zoom+1+args.Y/200*zoom)*.8
--					local region=mapWindow.Region:GetSelectedChoice()
--					local NS=math.floor((Grid[region].NSOffset-Y)*100)/100
--					local EW=math.floor((Grid[region].EWOffset+X)*100)/100

	local Y=(Grid[region].NSOffset-ns)*1.25*200/zoom-(mapWindow:GetHeight()/2-15)*zoomMult-200/zoom
	local X=(ew-Grid[region].EWOffset)*1.25*200/zoom-(mapWindow:GetWidth()/2-5)*zoomMult-200/zoom

	if Y<0 then Y=0 end
	if X<0 then X=0 end
	mapWindow.DoNotReload=true
	mapWindow.vGridScroll:SetValue(Y)
	mapWindow.hGridScroll:SetValue(X)
	mapWindow.vGridScroll.ValueChanged()
	mapWindow.hGridScroll.ValueChanged()
	mapWindow.DoNotReload=false
	mapWindow.ReloadGrid()
end
mapWindow.AddAnnotation = function(region, ns, ew, name, annotType, description)
	region=tonumber(region)
	ns=tonumber(ns)
	ew=tonumber(ew)
	if region~=nil and ns~=nil and ew~=nil then
		if description==nil or description=="" then description=name end
		local ns2=(Grid[region].NSOffset-ns)*1.25
		local ew2=(ew-Grid[region].EWOffset)*1.25
		local h=math.floor(ew2)
		local v=math.floor(ns2)
		local x=(ew2-h)*200
		local y=(ns2-v)*200

		if Annotations[region]==nil then Annotations[region]={} end
		if Annotations[region][v]==nil then Annotations[region][v]={} end
		if Annotations[region][v][h]==nil then Annotations[region][v][h]={} end
-- *** id should really be a unique ID, not just unique within the cell
		local id=#Annotations[region][v][h]+1
		local isDup=false
		for k,v in pairs(Annotations[region][v][h]) do
			if v.X==x and v.Y==y and v.Name==name then
				isDup=true
				break
			end
		end
		if not isDup then
			table.insert(Annotations[region][v][h],{["id"]=id,["Y"]=y,["X"]=x,["Name"]=name,["Type"]=annotType,["Description"]=description})
			-- refresh the grid
			mapWindow.ReloadGrid()
		else
			Turbine.Shell.WriteLine("An annotation named "..name.." already exists at ("..ns..", "..ew..").")
		end
	end
end

mapWindow.shellCmd.GetHelp = function(sender, cmd)
	return("usage:\n  /MapMaker show\n       Displays the MapMaker window.");
end
mapWindow.shellCmd.GetShortHelp = function(sender, cmd)
	return("MapMaker - usage: /MapMaker show");
end
mapWindow.SizeChanged=function()
	local newWidth,newHeight=mapWindow:GetSize()
	mapWindow.mapViewPort:SetSize(newWidth-10,newHeight-30);
	mapWindow.hGridScroll:SetWidth(newWidth-10);
	mapWindow.hGridScroll:SetTop(newHeight-10);
	mapWindow.vGridScroll:SetHeight(newHeight-30);
	mapWindow.vGridScroll:SetLeft(newWidth-10);
	mapWindow.SizeTab:SetPosition(newWidth-10,newHeight-10)
	mapWindow.Hourglass:SetPosition(newWidth/2-16,newHeight/2-16)
	mapWindow.CloseButton:SetLeft(newWidth-17)
end
mapWindow.AnnotCmds={}
mapWindow.AnnotLoaderState=0
mapWindow.AnnotLoader.Update=function()
	if mapWindow.AnnotLoaderState==0 then
		mapWindow.AnnotLoaderState=1
		Turbine.PluginManager.LoadPlugin("TMAnnot");
	else
		if Turbine.Shell.IsCommand("0TMAC") then
			for k,v in ipairs(mapWindow.AnnotCmds) do
				Turbine.Shell.RemoveCommand(v)
				mapWindow.AnnotCmds[k]=nil
			end
			local parms
			local cmds=Turbine.Shell.GetCommands();
			if cmds~=nil and type(cmds)=="table" then
				table.sort(cmds,function(arg1,arg2)if arg1<arg2 then return(true) end end);
				for cmdIndex=0,#cmds do
					if cmds[cmdIndex]~=nil then
						if cmdIndex>1 and cmds[cmdIndex]>="1" then
							break
						else
							parms=string.match(cmds[cmdIndex],"0TMA_(.*)");
							if parms~=nil then
								local tmpAnnot=string.split(parms,"_")
								local region=tonumber(tmpAnnot[1])
								local h=tonumber(tmpAnnot[2])
								local v=tonumber(tmpAnnot[3])
								local x=tonumber(tmpAnnot[4])
								local y=tonumber(tmpAnnot[5])
								local annotType=tonumber(tmpAnnot[6])
								local name=string.decode(tmpAnnot[7])
								local description=string.decode(tmpAnnot[8])
								if description==nil or description=="" then description=name end
								if Annotations[region]==nil then Annotations[region]={} end
								if Annotations[region][v]==nil then Annotations[region][v]={} end
								if Annotations[region][v][h]==nil then Annotations[region][v][h]={} end
								local id=#Annotations[region][v][h]+1
								table.insert(Annotations[region][v][h],{["id"]=id,["Y"]=y,["X"]=x,["Name"]=name,["Type"]=annotType,["Description"]=description})
							end
						end
					end
				end
			end

			-- unload the TMAnnot plugin
			pcall(Turbine.PluginManager.UnloadScriptState,"TMAnnot")
			mapWindow.AnnotLoaderState=0
			mapWindow.AnnotLoader:SetWantsUpdates(false)
			mapWindow.RefreshAnnotations()
		end
	end
end

if not Settings.LoadMinimized then
	mapWindow:SetVisible(true);
end
mapWindow:SetZOrder(2);
optionsWindow:SetZOrder(3);

usageWindow=Turbine.UI.Window()
usageWindow:SetSize(displayWidth,displayHeight)
usageWindow:SetVisible(false)
usageWindow:SetBackColor(Turbine.UI.Color(.1,.1,.1))
usageWindow:SetZOrder(3)

usageWindow.Title=Turbine.UI.Label()
usageWindow.Title:SetParent(usageWindow)
usageWindow.Title:SetSize(displayWidth-32,16)
usageWindow.Title:SetPosition(16,0)
usageWindow.Title:SetFont(Turbine.UI.Lotro.Font.Verdana14)
usageWindow.Title:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)

usageWindow.CloseButton=Turbine.UI.Control()
usageWindow.CloseButton:SetParent(usageWindow)
usageWindow.CloseButton:SetSize(16,16)
usageWindow.CloseButton:SetPosition(usageWindow:GetWidth()-16,0)
usageWindow.CloseButton:SetBackground(0x41000196)
usageWindow.CloseButton.MouseClick=function()
	usageWindow:SetVisible(false)
end
usageWindow.CloseButton.MouseEnter=function()
	usageWindow.CloseButton:SetBackground(0x41000198)
end
usageWindow.CloseButton.MouseLeave=function()
	usageWindow.CloseButton:SetBackground(0x41000196)
end
usageWindow.Display=Turbine.UI.Control()
usageWindow.Display:SetParent(usageWindow)
usageWindow.Display:SetSize(displayWidth,displayHeight-16)
usageWindow.Display:SetPosition(0,16)
usageWindow.DisplayOverlay=Turbine.UI.Control()
usageWindow.DisplayOverlay:SetParent(usageWindow)
usageWindow.DisplayOverlay:SetPosition(0,16)
usageWindow.DisplayOverlay:SetZOrder(1)
usageWindow.UpdateUsage=function()
	local region=mapWindow.Region:GetSelectedChoice()
	usageWindow.Title:SetText("Overview Map - Region "..tostring(region).." - Click on Overview to display location")
	-- determine size preserving aspect ration
	usageWindow.Display:SetStretchMode(0)
	usageWindow.Display:SetSize(1,1)
	local success,result=pcall(Turbine.UI.Control.SetBackground,usageWindow.Display,resourcePath.."R"..tostring(region).."_Background.jpg")
	if not success then
		usageWindow.Display:SetBackground(nil)
	else
		usageWindow.Display:SetStretchMode(2)
		usageWindow.Display:SetSize(usageWindow.Display:GetSize())
		local newWidth,newHeight=usageWindow.Display:GetSize()
		local maxWidth=displayWidth
		local maxHeight=displayHeight-16
		if newWidth/newHeight>maxWidth/maxHeight then
			-- width determines max size
			newHeight=math.floor(maxWidth*newHeight/newWidth)
			newWidth=maxWidth
		else
			newWidth=math.floor(maxHeight*newWidth/newHeight)
			newHeight=maxHeight
		end

		usageWindow.Display:SetSize(newWidth,newHeight)
		usageWindow.Display:SetStretchMode(1)
		local success,result=pcall(Turbine.UI.Control.SetBackground,usageWindow.Display,resourcePath.."R"..tostring(region).."_Background.jpg")
		usageWindow.DisplayOverlay:SetSize(newWidth,newHeight)
	end
end
usageWindow.DisplayOverlay.MouseClick=function(sender,args)
	-- determine x,y coordinates based on percent of MaxCol,MaxRow
	local region=mapWindow.Region:GetSelectedChoice()
	local maxX=MaxCol[region]
	local maxY=MaxRow[region]
	local x=maxX*(args.X+1)/usageWindow.DisplayOverlay:GetWidth()*.8
	local y=maxY*(args.Y+1)/usageWindow.DisplayOverlay:GetHeight()*.8

	local NS=Grid[region].NSOffset-y
	local EW=Grid[region].EWOffset+x
	mapWindow.ShowLocation(region,NS,EW)
	usageWindow:SetVisible(false)
end
debugWindow:SetZOrder(2)
mapWindow.ShowLocation(Settings.LastRegion,Settings.LastNS,Settings.LastEW)
